#pragma once

#include "font.h"
#include "icon.h"
#include "keyboard.h"
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#define SCREEN_WIDTH  1024
#define SCREEN_HEIGHT 768
#define FONT_WIDTH    13
#define FONT_HEIGHT   17
#define VGA_WIDTH     (SCREEN_WIDTH / FONT_WIDTH)
#define VGA_HEIGHT    (SCREEN_HEIGHT / FONT_HEIGHT)
#define TERM_COUNT    10

struct screen {
	size_t row;
	size_t column;
	uint32_t fg_color;
	uint32_t bg_color;
	uint8_t buffer[VGA_WIDTH * VGA_HEIGHT];
	uint32_t default_color;
	struct icon *background;
	struct font *font;
	char line[256];
};

typedef enum {
	VGA_COLOR_BLACK = 0,
	VGA_COLOR_BLUE = 1,
	VGA_COLOR_GREEN = 2,
	VGA_COLOR_CYAN = 3,
	VGA_COLOR_RED = 4,
	VGA_COLOR_MAGENTA = 5,
	VGA_COLOR_BROWN = 6,
	VGA_COLOR_LIGHT_GREY = 7,
	VGA_COLOR_DARK_GREY = 8,
	VGA_COLOR_LIGHT_BLUE = 9,
	VGA_COLOR_LIGHT_GREEN = 10,
	VGA_COLOR_LIGHT_CYAN = 11,
	VGA_COLOR_LIGHT_RED = 12,
	VGA_COLOR_LIGHT_MAGENTA = 13,
	VGA_COLOR_LIGHT_YELLOW = 14,
	VGA_COLOR_WHITE = 15,
} vga_color;

enum cursor_direction { LEFT, RIGHT, UP, DOWN };

void terminal_initialize(void);
void terminal_set_bg_color(uint32_t color);
void terminal_set_fg_color(uint32_t color);
int terminal_putchar(char c);
int terminal_write(const char *data, size_t size);
int terminal_writestring(const char *data);
int terminal_writelong(long number);
void terminal_set_screen(int pos);
void terminal_clear(void);
void terminal_refresh(void);
struct key_event terminal_getkey(void);
void update_cursor(void);
void move_cursor(int direction);
void set_color_level(int level);
void terminal_set_default_fg_color(uint32_t fg_color);
void terminal_set_default_bg_color(uint32_t fg_color);
void terminal_change_default_fg_color(uint32_t color);
uint32_t terminal_get_default_color(void);
uint8_t terminal_get_char(int column, int row);
void terminal_remove_last_char(void);
