#include "alloc.h"
#include "debug.h"
#include "kpanic.h"
#include "kprintf.h"
#include "memory.h"

Zone *zones[3];

static void add_zone(Zone *zone, block_type_t type)
{
	// We put the zone at the beginning of the list
	if (zones[type]) {
		assert(zones[type] != zone);
		zone->next = zones[type];
		zones[type]->prev = zone;
	}
	zones[type] = zone;
}

static void new_block(Zone *zone, uint32_t zone_size)
{
	Block *new_block = (Block *)align_mem((uint32_t)zone + sizeof(Zone));

	// Metadata
	new_block->in_use = false;
	new_block->size = zone_size - sizeof(Zone) - sizeof(Block);
	new_block->sub_size = new_block->size;
	new_block->ptr = (Block *)((uint32_t)new_block + sizeof(Block));
	new_block->zone = zone;

	// Init future linked lists
	new_block->prev = NULL;
	new_block->prev_free = NULL;
	new_block->prev_used = NULL;
	new_block->next = NULL;
	new_block->next_free = NULL;
	new_block->next_used = NULL;

	if (zone->free) {
		zone->free->prev = new_block;
		zone->free->prev_free = new_block;
		new_block->next = zone->free;
		new_block->next_free = zone->free;
	}
	zone->free = new_block;
	assert(zone->free == new_block);
}

int new_vzone(block_type_t type, uint32_t size)
{
	void *heap = alloc_pages(size, NULL);
	if (heap == NULL) {
		kprintf(KERN_ERR "error: alloc_pages failed\n");
		return (-1);
	}

	Zone *zone = (Zone *)heap;
	zone->type = type;
	zone->size = size;
	zone->used = NULL;
	zone->next = NULL;
	zone->prev = NULL;

	new_block(zone, size);
	add_zone(heap, type);

	return (0);
}
